/*
 Project name:
       LIN Hall Click
 Copyright:
      (c) mikroElektronika, 2014.
 Revision History:
      20140822:
      - Initial release (RD);
 Description:
      LIN HALL click is a simple solution for adding a linear Hall sensor to your design. 
      It carries an MLX90242 linear Hall sensor IC (with built-in active error correction circuitry) and an MCP3201 12-bit ADC. 
      LIN HALL click communicates with the target board through the mikroBUS SPI lines (CS, SCK, MISO). 
      The board is designed to use either a 3.3V or 5V power supply.
      LIN HALL click, with its linear Hall sensor, outputs a signal that is proportional to the flux density of a magnetic field. 
      The output signal will increase when a north magnetic field is applied, while a south magnetic field will cause it to decrease. 
      The ratiometric output can be used to measure the strength of different magnets, or to measure the distance between the sensor 
      and a magnet (which can be used for linear or rotary position sensing by placing magnets on the target object). LCD shows data.

 Test configuration:
     MCU:             P32MX795F512L
                      http://ww1.microchip.com/downloads/en/DeviceDoc/61156F.pdf
     Dev.Board:       EasyPIC Fusion v7
                      http://www.mikroe.com/easypic-fusion/
     Oscillator:      XT-PLL, 80.000MHz
     ext. modules:    BI Hall Click
                      http://www.mikroe.com/click/bi-hall/
     SW:              mikroC PRO for PIC32
                      http://www.mikroe.com/mikroc/pic32/
 NOTES:
      - Place Lin Hall Click board in the mikroBUS socket 1.
      - Set development board J5 jumper according to SMD jumper on click board. By default it's soldered in the 3.3V position.
      - Turn On SW11-3 for TFT backlight.

*/

// HALL click module connections
sbit Chip_Select_Direction   at TRISC2_bit;
sbit Chip_Select             at LATC2_bit;
// end HALL click module connections

// TFT module connections
char TFT_DataPort at LATE;
sbit TFT_RST at LATD7_bit;
sbit TFT_BLED at LATD2_bit;
sbit TFT_RS at LATD9_bit;
sbit TFT_CS at LATD10_bit;
sbit TFT_RD at LATD5_bit;
sbit TFT_WR at LATD4_bit;
char TFT_DataPort_Direction at TRISE;
sbit TFT_RST_Direction at TRISD7_bit;
sbit TFT_BLED_Direction at TRISD2_bit;
sbit TFT_RS_Direction at TRISD9_bit;
sbit TFT_CS_Direction at TRISD10_bit;
sbit TFT_RD_Direction at TRISD5_bit;
sbit TFT_WR_Direction at TRISD4_bit;
// End TFT module connections

unsigned int measurement, value_old = 0;
char txt[4];


// Get ADC values
unsigned int getADC() {                          // Returns 0..4095
  unsigned int tmp;
  Chip_Select = 0;                               // Select MCP3201
  tmp = SPI3_Read(0) & 0x1F;                     // Dummy 0 byte out , read in upper bits - B11-B7
  tmp = tmp << 7;                                // Shift ADC value by 7
  tmp += SPI3_Read(0) >> 1;                      // Dummy 0 byte out , read in lower  bits, B6-B0 + B1
                                                 // and shift right by 1 to determine B1
  Chip_Select= 1;                                // Deselect MCP3201
  return tmp;                                    // Returns 12-bit ADC value
}

void DrawFrame() {
  TFT_Fill_Screen(CL_WHITE);
  TFT_Set_Pen(CL_BLACK, 1);
  TFT_Line(20, 220, 300, 220);
  TFT_LIne(20,  46, 300,  46);
  TFT_Set_Font(&TFT_defaultFont, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("UNI HALL Click  Board  Demo", 25, 14);
  TFT_Set_Font(&TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
  TFT_Write_Text("EasyPIC Fusion v7", 19, 223);
  TFT_Set_Font(&TFT_defaultFont, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("www.mikroe.com", 200, 223);
  TFT_Set_Font(&TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
  TFT_Write_Text("Value:", 80, 90);
}

void Display_Measurement_Value(unsigned long value)  {
  if (value_old != value) {                 // If old value and current value are not equal
    TFT_Set_Font(&TFT_defaultFont, CL_WHITE, FO_HORIZONTAL);
    TFT_Write_Text(txt, 180, 90);           // Display string on TFT
    WordToStr(value, txt);                 // Convert illuminance value to string
    Ltrim(&txt);                            // Trim leading spaces
    TFT_Set_Font(&TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Text(txt, 180, 90);           // Display string on TFT
  }
  value_old = value;
}

void main() {
  AD1PCFG     = 0xFFFF;                     // Turning all analog pins to digital

  TFT_BLED_Direction = 0;                   // Set TFT backlight pin as output
  TFT_Init_ILI9341_8bit(320, 240);          // Initialize TFT display
  TFT_BLED = 1;                             // Turn on TFT backlight

  measurement = 0;                          // Initialize the measurement variable

  Chip_Select = 1;                          // Deselect MCP3201
  Chip_Select_Direction = 0;                // Set chip select pin to be output

  // Initialize SPI1 module
  SPI3_Init();
  SPI_Set_Active(&SPI3_Read, &SPI3_Write);  // Sets the SPI3 module active

  DrawFrame();
  
  while (1) {
    measurement = getADC();                 // Get ADC result
    Display_Measurement_Value(measurement);
    Delay_ms(200);                          // Wait 200ms
  }
}